# encoding: UTF-8
###############################################
#
# Script sample from Alexander C. Schreyer's book
# "Architectural Design with SketchUp"
#
###############################################
#
# Creates a square panel with a sinusoidal hole pattern

module Odowhacker
  module HolePattern
    extend self
    @@SEGMENTS = 24
            
    def do_pattern
      model = Sketchup.active_model
      model.start_operation("Make Pattern", true)
      # Create group so geometry we create doesn't stick to existing
      ent = model.entities.add_group.entities
      
      width = 36 # Width of panel
      n = 10 # Number of circles in each direction
      s = width / (n+1).to_f # Spacing of circles
      w = 50
      
      # Add the square base for the panel
      face = ent.add_face( [0,0,0] , [width,0,0] , [width,width,0] , [0,width,0] )

      # Iterate in the panel plane
      n.times do |i|
        n.times do |j|
          # Create a group for each circle
          group = ent.add_group
          ents = group.entities
          # Add the circle
          vector = Z_AXIS
          centerpoint = [s+i*s, s+j*s, 0]
          radius = Math::sin( i / (n -1 ).to_f * Math::PI ) * s / 5.0 + 
            Math::sin( j / (n - 1).to_f * Math::PI ) * s / 5.0
          puts("i = #{i}, j = #{j}")
          puts("add_circle(#{centerpoint}, #{vector}, #{radius}, #{@@SEGMENTS})")
          edges = ents.add_circle(centerpoint , vector , radius , @@SEGMENTS)
          puts("edges.size #{edges.size}")

          if ( edges.size == @@SEGMENTS )
            edges[0].find_faces
            cface = edges[0].faces[0]
            cface.back_material = [
              (255 / ( n- 1  ) * i).round,
              (255 / ( n - 1 ) * j).round,
              (255 / ( n - 1 ) * j).round
            ]
            cface.pushpull -w
            group.explode
          else
            puts("Erasing group with #{edges.size} edges.")
            group.erase! # Remove tiny edges
          end #if
        end #n.times j
      end #n.times i
      model.commit_operation
    end #do_pattern
    
    ## Startup Block   
    this_file = Module::nesting[0].name << ':'<< File.basename(__FILE__)
    unless file_loaded?(this_file)
      menu = UI.menu("Plugins")
      menu.add_item("Make Pattern") {self::do_pattern}
      file_loaded(this_file)
    end #unless file_loaded

  end #HolePattern
end #Odowhacker